//Control Unit
//Kamyar Rafati & Naeem Esfahani

module Control (Op, Branch, BranchInvert, ControlData, PCJump, reset, clk);
	input[5:0]	Op;
	input		clk, reset;
	output		Branch, BranchInvert, PCJump;
	output[7:0]	ControlData;	
	reg			Branch, BranchInvert, PCJump;
	reg[7:0]	ControlData; 


	always @(posedge clk or Op)
		if(reset == 0)
		case(Op)
			0: 	//R-Type
			begin
				Branch = 0;
				BranchInvert = 0;
				// Control Data:  RegWrite MemToReg MemRead MemWrite ALUSrc ALUOp(2) RegDst             
				ControlData = 8'b10000101;
				PCJump = 0;
			end

			2: //Jump
			begin
				Branch = 0;
				BranchInvert = 0;
				// Control Data:  RegWrite MemToReg MemRead MemWrite ALUSrc ALUOp(2) RegDst             
				ControlData = 8'b00000000;
				PCJump = 1;
			end

			4: //BEQ
			begin
				Branch = 1;
				BranchInvert = 0;
				// Control Data:  RegWrite MemToReg MemRead MemWrite ALUSrc ALUOp(2) RegDst             
				PCJump = 0;
				ControlData = 8'b00000011;
			end

			5: //BNE
			begin
				Branch = 1;
				BranchInvert = 1;
				// Control Data:  RegWrite MemToReg MemRead MemWrite ALUSrc ALUOp(2) RegDst             
				ControlData = 8'b00000011;
				PCJump = 0;
			end

			8:	//addi
			begin
				Branch = 0;
				BranchInvert = 0;
				// Control Data:  RegWrite MemToReg MemRead MemWrite ALUSrc ALUOp(2) RegDst             
				ControlData = 8'b10001000;
				PCJump = 0;
			end

			35: //LW
			begin
				Branch = 0;
				BranchInvert = 0;
				// Control Data:  RegWrite MemToReg MemRead MemWrite ALUSrc ALUOp(2) RegDst             
				ControlData = 8'b11101000;
				PCJump = 0;
			end

			43: //SW
			begin
				Branch = 0;
				BranchInvert = 0;
				// Control Data:  RegWrite MemToReg MemRead MemWrite ALUSrc ALUOp(2) RegDst             
				ControlData = 8'b00011000;
				PCJump = 0;
			end
		endcase
		else
		begin
			Branch = 0;
			BranchInvert = 0;
			ControlData = 8'b00000000;
		end
endmodule
